@@ -1,161 +1,13 @@
+import itertools
+
+import utils
 import ipmath
 
 masks = xrange(33)
 
-def validateMask(mask):
-    '''
-    Validate the variable length subnet masking value.
-
-    >>> validateMask(50)
-    Traceback (most recent call last):
-    ValueError: The subnet mask (prefix length) must be between 0 and 32, inclusive.
-    >>> validateMask('45')
-    Traceback (most recent call last):
-    ValueError: The subnet mask (prefix length) must be between 0 and 32, inclusive.
-    >>> validateMask(32)
-    >>> validateMask('27')
-    >>> validateMask(17)
-    >>> validateMask('0')
-    >>> validateMask(-1)
-    Traceback (most recent call last):
-    ValueError: The subnet mask (prefix length) must be between 0 and 32, inclusive.
-    >>> validateMask('-10')
-    Traceback (most recent call last):
-    ValueError: The subnet mask (prefix length) must be between 0 and 32, inclusive.
-    '''
-    if 0 > int(mask)  or int(mask) > 32:
-        msg = ('The subnet mask (prefix length) must be between 0 and ' +
-            '32, inclusive.')
-        raise ValueError, msg
-
-def _getLargest(start, excluded):
-    '''
-    We work our way from smallest mask to largest (largest network to
-    smallest), so the first network we get to that doesn't have the IP that we
-    want to exclude will actually be the largest network without it.
-
-    >>> start = '172.31.0.0'
-    >>> excluded = '172.31.0.10'
-    >>> c = _getLargest(start, excluded)
-    >>> c
-    172.31.0.0/29
-    >>> c.getHostRange()
-    (172.31.0.0, 172.31.0.7)
-
-    >>> start = '192.168.4.112'
-    >>> excluded = '192.168.4.128'
-    >>> c = _getLargest(start, excluded)
-    >>> c
-    192.168.4.112/28
-    >>> c.getHostRange()
-    (192.168.4.112, 192.168.4.127)
-    '''
-    for mask in masks:
-        c = CIDR('%s/%s' % (start, mask))
-        if c.getHostRange()[0] != CIDR(start):
-            continue
-        if c and CIDR(excluded) not in Networks([c]):
-            return c
-
-def getLargest(start, excludedList):
-    '''
-    Get the largest possible network that doesn't include the given IP
-    addressed.
-
-    >>> startIP = '10.0.0.0'
-    >>> exclude = ['10.0.6.50', '10.0.10.0']
-    >>> c = getLargest(startIP, exclude)
-    >>> c
-    10.0.0.0/22
-    >>> c.getHostRange()
-    (10.0.0.0, 10.0.3.255)
-
-    >>> startIP = '10.0.7.0'
-    >>> exclude = ['10.0.6.50', '10.0.10.0']
-    >>> c = getLargest(startIP, exclude)
-    >>> c
-    10.0.7.0/24
-    >>> c.getHostRange()
-    (10.0.7.0, 10.0.7.255)
-
-    # let's test some tricky stuff
-    >>> exclude = ['192.168.4.67', '192.168.4.127']
-    >>> c = getLargest('192.168.4.64', exclude)
-    >>> c
-    192.168.4.64/31
-    >>> c.getHostRange()
-    (192.168.4.64, 192.168.4.65)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.66', exclude)
-    >>> c
-    192.168.4.66
-    >>> c.getHostRange()
-    (192.168.4.66, 192.168.4.66)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.68', exclude)
-    >>> c
-    192.168.4.68/30
-    >>> c.getHostRange()
-    (192.168.4.68, 192.168.4.71)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.71', exclude)
-    >>> c
-    192.168.4.71
-    >>> c.getHostRange()
-    (192.168.4.71, 192.168.4.71)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.72', exclude)
-    >>> c
-    192.168.4.72/29
-    >>> c.getHostRange()
-    (192.168.4.72, 192.168.4.79)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.80', exclude)
-    >>> c
-    192.168.4.80/28
-    >>> c.getHostRange()
-    (192.168.4.80, 192.168.4.95)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-
-    >>> c = getLargest('192.168.4.96', exclude)
-    >>> c
-    192.168.4.96/28
-    >>> c.getHostRange()
-    (192.168.4.96, 192.168.4.111)
-    >>> set(exclude).intersection(c.getIPs())
-    set([])
-    '''
-    candidates = []
-    def compareIPs(x, y):
-        x = CIDR(x)
-        y = CIDR(y)
-        return cmp(x.getOctetTuple(), y.getOctetTuple())
-    excludedList.sort(compareIPs)
-    for excl in excludedList:
-        if CIDR(start) > CIDR(excl):
-            continue
-        candidates.append(_getLargest(start, excl))
-    # We want the reverse sorted order because the larger masks are actually
-    # the smaller networks; if we've got a list of IPs we're not including, we
-    # want to start with the smallest first.
-    #candidates.sort(compareIPs)
-    if candidates:
-        return candidates[0]
-
 def getSpanningBlocks(start, end):
     '''
-    This is a special case of the getLargest function, where the IP address to
+    This is a special case of the getLargestBlock function, where the IP address to
     be excluded is end + 1.
 
     >>> start = '172.31.0.0'
@@ -197,37 +49,139 @@ def getSpanningBlocks(start, end):
     nets = []
     current = start
     excluded = ipmath.int2octets(ipmath.octets2int(end) + 1)
-    nets.append(_getLargest(start, excluded))
+    nets.append(_getLargestBlock(start, excluded))
     while CIDR(current) <= CIDR(end):
         current = nets[-1].getHostRange()[-1].getIP()
         current = ipmath.int2octets(ipmath.octets2int(current) + 1)
-        net = _getLargest(current, excluded)
+        net = _getLargestBlock(current, excluded)
         if net:
             nets.append(net)
     return nets
 
-def incrementIP(ip):
+def _getLargestBlock(start, excluded):
     '''
-    >>> ip = '192.168.4.1'
-    >>> incrementIP(ip)
-    '192.168.4.2'
-    >>> ip = '10.0.5.255'
-    >>> incrementIP(ip)
-    '10.0.6.0'
-    '''
-    current = ipmath.octets2int(ip) + 1
-    return ipmath.int2octets(current)
+    We work our way from smallest mask to largest (largest network to
+    smallest), so the first network we get to that doesn't have the IP that we
+    want to exclude will actually be the largest network without it.
 
-def getIPList(startIP, endIP):
+    >>> start = '172.31.0.0'
+    >>> excluded = '172.31.0.10'
+    >>> c = _getLargestBlock(start, excluded)
+    >>> c
+    172.31.0.0/29
+    >>> c.getHostRange()
+    (172.31.0.0, 172.31.0.7)
+
+    >>> start = '192.168.4.112'
+    >>> excluded = '192.168.4.128'
+    >>> c = _getLargestBlock(start, excluded)
+    >>> c
+    192.168.4.112/28
+    >>> c.getHostRange()
+    (192.168.4.112, 192.168.4.127)
     '''
-    >>> ips = getIPList('172.16.0.0', '172.16.0.8')
-    >>> len(ips)
-    9
-    >>> ips
-    ['172.16.0.0', '172.16.0.1', '172.16.0.2', '172.16.0.3', '172.16.0.4', '172.16.0.5', '172.16.0.6', '172.16.0.7', '172.16.0.8']
+    for mask in masks:
+        c = CIDR('%s/%s' % (start, mask))
+        if c.getHostRange()[0] != CIDR(start):
+            continue
+        if c and CIDR(excluded) not in Networks([c]):
+            return c
+
+def getLargestBlock(start, excludedList):
     '''
-    ints = range(ipmath.octets2int(startIP), ipmath.octets2int(endIP) + 1)
-    return [ipmath.int2octets(x) for x in ints]
+    Get the largest possible network that doesn't include the given IP
+    addressed.
+
+    >>> startIP = '10.0.0.0'
+    >>> exclude = ['10.0.6.50', '10.0.10.0']
+    >>> c = getLargestBlock(startIP, exclude)
+    >>> c
+    10.0.0.0/22
+    >>> c.getHostRange()
+    (10.0.0.0, 10.0.3.255)
+
+    >>> startIP = '10.0.7.0'
+    >>> exclude = ['10.0.6.50', '10.0.10.0']
+    >>> c = getLargestBlock(startIP, exclude)
+    >>> c
+    10.0.7.0/24
+    >>> c.getHostRange()
+    (10.0.7.0, 10.0.7.255)
+
+    # let's test some tricky stuff
+    >>> exclude = ['192.168.4.67', '192.168.4.127']
+    >>> c = getLargestBlock('192.168.4.64', exclude)
+    >>> c
+    192.168.4.64/31
+    >>> c.getHostRange()
+    (192.168.4.64, 192.168.4.65)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.66', exclude)
+    >>> c
+    192.168.4.66
+    >>> c.getHostRange()
+    (192.168.4.66, 192.168.4.66)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.68', exclude)
+    >>> c
+    192.168.4.68/30
+    >>> c.getHostRange()
+    (192.168.4.68, 192.168.4.71)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.71', exclude)
+    >>> c
+    192.168.4.71
+    >>> c.getHostRange()
+    (192.168.4.71, 192.168.4.71)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.72', exclude)
+    >>> c
+    192.168.4.72/29
+    >>> c.getHostRange()
+    (192.168.4.72, 192.168.4.79)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.80', exclude)
+    >>> c
+    192.168.4.80/28
+    >>> c.getHostRange()
+    (192.168.4.80, 192.168.4.95)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+
+    >>> c = getLargestBlock('192.168.4.96', exclude)
+    >>> c
+    192.168.4.96/28
+    >>> c.getHostRange()
+    (192.168.4.96, 192.168.4.111)
+    >>> set(exclude).intersection(c.getIPs())
+    set([])
+    '''
+    candidates = []
+    def compareIPs(x, y):
+        x = CIDR(x)
+        y = CIDR(y)
+        return cmp(x.getOctetTuple(), y.getOctetTuple())
+    excludedList.sort(compareIPs)
+    for excl in excludedList:
+        if CIDR(start) > CIDR(excl):
+            continue
+        candidates.append(_getLargestBlock(start, excl))
+    # We want the reverse sorted order because the larger masks are actually
+    # the smaller networks; if we've got a list of IPs we're not including, we
+    # want to start with the smallest first.
+    #candidates.sort(compareIPs)
+    if candidates:
+        return candidates[0]
 
 class CIDR(object):
     '''
@@ -306,7 +260,7 @@ class CIDR(object):
         if len(net) == 2:
             net, mask = net
             mask = int(mask)
-            validateMask(mask)
+            utils.validateMask(mask)
         elif len(net) > 2:
             msg = "There appear to be too many '/' in your network notation."
             raise ValueError, msg
@@ -557,11 +511,8 @@ class CIDR(object):
         '''
         An iterator for all the IP addresses in the CIDR object's range.
         '''
-        start, stop = self.getHostRange()
-        start = ipmath.octets2int(str(start))
-        stop = ipmath.octets2int(str(stop))
-        for x in range(start, stop + 1):
-            yield ipmath.int2octets(x)
+        start, stop = [x.getIP() for x in self.getHostRange()]
+        return utils.iterIPs(start, stop)
 
     def getIPs(self):
         '''
@@ -658,6 +609,42 @@ class Networks(list):
             return True
         return False
 
+    def getHostCount(self):
+        '''
+        Return the total number of hosts for all blocks defined in the Networks
+        instance.
+
+        >>> blocks = ['172.16.0.0/16', '172.29.13.0/24', '10.1.24.0/26']
+        >>> nets = Networks([CIDR(x) for x in blocks])
+        >>> obtained = nets.getHostCount()
+        >>> obtained
+        65856
+        >>> counts = [CIDR(x).getHostCount() for x in blocks]
+        >>> counts
+        [65536, 256, 64]
+        >>> obtained == sum(counts)
+        True
+        '''
+        return sum([x.getHostCount() for x in self])
+
+    def iterIPs(self):
+        '''
+        Iterate through the IP addresses for each block in the collection.
+
+        >>> ips = ['192.168.4.44/31', '172.16.5.5/32', '10.2.1.1/30']
+        >>> nets = Networks([CIDR(x) for x in ips])
+        >>> for host in nets.iterIPs():
+        ...   print host
+        192.168.4.44
+        192.168.4.45
+        172.16.5.5
+        10.2.1.0
+        10.2.1.1
+        10.2.1.2
+        10.2.1.3
+        '''
+        return itertools.chain(*[x.iterIPs() for x in self])
+
     def assembleBlocks(self, startIP, endIP):
         '''
         Build a collection of network, the sum of which exactly span
@@ -785,7 +772,7 @@ class Networks(list):
         192.168.5.128/25
 
         # let's use sets for checking the results
-        >>> all = set(getIPList(startIP, endIP))
+        >>> all = set(utils.getIPs(startIP, endIP))
         >>> expected = all.difference(exclude)
         >>> obtained = []
         >>> for net in nets:
@@ -805,13 +792,13 @@ class Networks(list):
         current = startIP
         while ipmath.octets2int(current) < ipmath.octets2int(endIP):
             if current in listOfIPs:
-                current = incrementIP(current)
+                current = ipmath.incrementIP(current)
                 continue
-            cidr = getLargest(current, listOfIPs)
+            cidr = getLargestBlock(current, listOfIPs)
             if cidr:
                 self.append(cidr)
                 ip = cidr.getHostRange()[-1].getIP()
-                current = incrementIP(ip)
+                current = ipmath.incrementIP(ip)
             else:
                 self.assembleBlocks(current, endIP)
                 current = endIP
@@ -839,7 +826,7 @@ class Networks(list):
         '''
         startIP = listOfIPs[0]
         endIP = listOfIPs[-1]
-        all = set(getIPList(startIP, endIP))
+        all = set(utils.getIPs(startIP, endIP))
         excluded = list(all.symmetric_difference(listOfIPs))
         self.withoutIPs(startIP, endIP, excluded)
 
